# -*- coding: utf-8 -*-
from __future__ import absolute_import, print_function, unicode_literals

from itertools import count, islice, tee
from logging import getLogger

from requests import codes

from prospyr import connection, exceptions

logger = getLogger(__name__)


class ResultSet(object):
    """
    Immutable, lazy search results.
    """

    def __init__(self, resource_cls, params=None, order_field=None,
                 order_dir='asc', using='default', page_size=100):
        self._params = params or {}
        self._order_field = order_field
        self._order_dir = order_dir
        self._resource_cls = resource_cls
        self._using = using
        self._results = self._results_generator()
        self._page_size = page_size

    def all(self):
        return self.filter()

    def filter(self, **query):
        new_params = self._params.copy()
        new_params.update(query)
        return ResultSet(params=new_params, using=self._using,
                         resource_cls=self._resource_cls,
                         order_field=self._order_field,
                         order_dir=self._order_dir, page_size=self._page_size)

    def order_by(self, field):
        dir = 'asc'
        if field.startswith('-'):
            dir, field = 'desc', field[1:]
        available = self._resource_cls.Meta.order_fields
        if field not in available:
            raise ValueError(
                'Cannot sort by `{field}`; try one of {valid}'
                .format(field=field, valid=', '.join(sorted(available)))
            )
        return ResultSet(params=self._params, using=self._using,
                         resource_cls=self._resource_cls, order_dir=dir,
                         order_field=field, page_size=self._page_size)

    @property
    def _conn(self):
        return connection.get(self._using)

    @property
    def _url(self):
        path = self._resource_cls.Meta.search_path
        return self._conn.build_absolute_url(path)

    def _build_query(self):
        query = dict(page_size=self._page_size, **self._params)
        if self._order_field:
            query.update({
                'sort_by': self._order_field,
                'sort_direction': self._order_dir,
            })
        return query

    def _results_generator(self):
        """
        Return Resource instances by querying ProsperWorks.

        You should not normally need to use this method.
        """
        query = self._build_query()

        for query['page_number'] in count(1):
            resp = self._conn.post(self._url, json=query)
            if resp.status_code != codes.ok:
                raise exceptions.ApiError(resp.status_code, resp.text)

            # no (more) results
            page_data = resp.json()
            logger.debug('%s results on page %s of %s',
                         len(page_data), query['page_number'], self._url)

            # 200 OK (not 404) and empty results if no more results
            if not page_data:
                break

            for data in page_data:
                yield self._resource_cls.from_api_data(data)

            # detect last page of results
            if len(page_data) < self._page_size:
                break

    def __iter__(self):
        """
        All resource instances matching this search. Results are cached.

        Depending on page size, this could result in many requests.
        """
        self._results, cpy = tee(self._results)
        return cpy

    def __getitem__(self, index):
        """
        Fetch the nth of sliceth item from cache or ProsperWorks.

        Fetching item N involves fetching items 0 through N-1. Depending on
        page size and N, this could be many requests.
        """
        negative = (
            type(index) is slice and (
                (index.start is not None and index.start < 0) or
                (index.stop is not None and index.stop < 0)
            ) or
            type(index) is not slice and index < 0
        )
        if negative:
            raise IndexError('ResultSet does not support negative indexing')

        self._results, cpy = tee(self._results)
        if type(index) is slice:
            return list(islice(cpy, index.start, index.stop, index.step))
        else:
            try:
                return next(islice(cpy, index, index+1))
            except StopIteration:
                raise IndexError('ResultSet index out of range')

    def __repr__(self):
        # show up to 5 results, then elipses. 6 results are fetched to
        # accomodate the elipses logic.
        first_6 = [str(r) for r in self[:6]]
        truncated = len(first_6) == 6
        if truncated:
            first_6 = first_6[0:5] + ['...']

        return '<ResultSet: {results}>'.format(
            results=', '.join(first_6)
        )



# -*- coding: utf-8 -*-

from __future__ import absolute_import, print_function, unicode_literals

import functools
import re

import requests
from urlobject import URLObject
from urlobject.path import URLPath

from prospyr.cache import InMemoryCache
from prospyr.exceptions import MisconfiguredError
from prospyr.util import seconds

_connections = {}
_default_url = 'https://api.prosperworks.com/developer_api/'


def connect(email, token, url=_default_url, name='default', cache=None):
    """
    Create a connection to ProsperWorks using credentials `email` and `token`.

    The new connection is returned. It can be retrieved later by name using
    `prospyr.connection.get`. By default the connection is named 'default'. You
    can provide a different name to maintain multiple connections to
    ProsperWorks.

    By default an in-memory URL cache is used. Argue
    cache=prospyr.cache.NoOpCache() to disable caching.
    """
    if name in _connections:
        existing = _connections[name]
        raise ValueError(
            '`{name}` is already connected using account '
            '"{email}"'.format(name=name, email=existing.email)
        )

    validate_url(url)

    conn = Connection(url, email, token, cache=cache, name=name)
    _connections[name] = conn
    return conn


def get(name='default'):
    """
    Fetch a ProsperWorks connection by name.

    If you did not argue a name at connection time, the connection will be
    named 'default'.
    """
    try:
        return _connections[name]
    except KeyError:
        if name == 'default':
            msg = ('There is no default connection. '
                   'First try prospyr.connect(...)')
        else:
            msg = ('There is no connection named "{name}". '
                   'First try prospyr.connect(..., name="{name}")')
            msg = msg.format(name=name)
        raise MisconfiguredError(msg)


def validate_url(url):
    """
    True or MisconfiguredError if `url` is invalid.
    """
    uo = URLObject(url)
    if not uo.scheme or uo.scheme not in {'http', 'https'}:
        raise MisconfiguredError('ProsperWorks API URL `%s` must include a '
                                 'scheme (http, https)' % url)
    if not uo.hostname:
        raise MisconfiguredError('ProsperWorks API URL `%s` must include a '
                                 'hostname' % url)
    if re.search('/v\d', url):
        raise MisconfiguredError('ProsperWorks API URL `%s` should not '
                                 'include a "version" path segment' % url)

    return True


def url_join(base, *paths):
    """
    Append `paths` to `base`. Path resets on each absolute path.

    Like os.path.join, but for URLs.
    """
    if not hasattr(base, 'add_path'):
        base = URLObject(base)

    for path in paths:
        path = URLPath(path)
        base = base.add_path(path)
    return base


class Connection(object):

    def __init__(self, url, email, token, name='default', version='v1',
                 cache=None):
        self.session = Connection._get_session(email, token)
        self.email = email
        self.base_url = URLObject(url)
        self.api_url = self.base_url.add_path_segment(version)
        self.cache = InMemoryCache() if cache is None else cache
        self.name = name

    def http_method(self, method, url, *args, **kwargs):
        """
        Send HTTP request with `method` to `url`.
        """
        method_fn = getattr(self.session, method)
        return method_fn(url, *args, **kwargs)

    def build_absolute_url(self, path):
        """
        Resolve relative `path` against this connection's API url.
        """
        return url_join(self.api_url, path)

    @staticmethod
    def _get_session(email, token):
        session = requests.Session()
        defaults = {
            'X-PW-Application': 'developer_api',
            'X-PW-AccessToken': token,
            'X-PW-UserEmail': email,
            'Accept': 'application/json',
            'Content-Type': 'application/json',
        }
        session.headers.update(defaults)
        return session

    def __getattr__(self, name):
        """
        Turn HTTP verbs into http_method calls so e.g. conn.get(...) works.

        Note that 'get' and 'delete' are special-cased to handle caching
        """
        methods = 'post', 'put', 'patch', 'options'
        if name in methods:
            return functools.partial(self.http_method, name)
        return super(Connection, self).__getattr__(name)

    def get(self, url, *args, **kwargs):
        cached = self.cache.get(url)
        if cached is None:
            cached = self.http_method('get', url, *args, **kwargs)
            self.cache.set(url, cached, max_age=seconds(minutes=5))
        return cached

    def delete(self, url, *args, **kwargs):
        resp = self.http_method('delete', url, *args, **kwargs)
        if resp.ok:
            self.cache.clear(url)
        return resp



# -*- coding: utf-8 -*-

from __future__ import absolute_import, print_function, unicode_literals

from logging import getLogger

from requests import codes

from prospyr.exceptions import ApiError

logger = getLogger(__name__)


class Creatable(object):
    """
    Allows creation of a Resource. Should be mixed in with that class.
    """

    # pworks uses 200 OK for creates. 201 CREATED is here through optimism.
    _create_success_codes = {codes.created, codes.ok}

    def create(self, using='default'):
        """
        Create a new instance of this Resource. True on success.
        """
        if hasattr(self, 'id'):
            raise ValueError(
                '%s cannot be created; it already has an id' % self
            )
        conn = self._get_conn(using)
        path = self.Meta.create_path
        resp = conn.post(conn.build_absolute_url(path), json=self._raw_data)

        if resp.status_code in self._create_success_codes:
            data = self._data_from_resp(resp)
            self._set_fields(data)
            return True
        elif resp.status_code == codes.unprocessable_entity:
            error = resp.json()
            raise ValueError(error['message'])
        else:
            raise ApiError(resp.status_code, resp.text)


class Readable(object):
    """
    Allows reading of a Resource. Should be mixed in with that class.
    """

    _read_success_codes = {codes.ok}

    def read(self, using='default'):
        """
        Read this Resource from remote API. True on success.
        """
        logger.debug('Connected using %s', using)
        if getattr(self, 'id', None) is None:
            raise ValueError('%s must be saved before it is read' % self)
        conn = self._get_conn(using)
        path = self.Meta.detail_path.format(id=self.id)
        resp = conn.get(conn.build_absolute_url(path))
        if resp.status_code not in self._read_success_codes:
            raise ApiError(resp.status_code, resp.text)

        data = self._data_from_resp(resp)
        self._set_fields(data)
        return True


class Updateable(object):
    """
    Allows updating a Resource. Should be mixed in with that class.
    """

    _update_success_codes = {codes.ok}

    def update(self, using='default'):
        """
        Update this Resource. True on success.
        """
        if getattr(self, 'id', None) is None:
            raise ValueError('%s cannot be deleted before it is saved' % self)

        # can't update IDs
        data = self._raw_data
        data.pop('id')

        conn = self._get_conn(using)
        path = self.Meta.detail_path.format(id=self.id)
        resp = conn.put(conn.build_absolute_url(path), json=data)
        if resp.status_code in self._update_success_codes:
            return True
        elif resp.status_code == codes.unprocessable_entity:
            error = resp.json()
            raise ValueError(error['message'])
        else:
            raise ApiError(resp.status_code, resp.text)


class Deletable(object):
    """
    Allows deletion of a Resource. Should be mixed in with that class.
    """

    _delete_success_codes = {codes.ok}

    def delete(self, using='default'):
        """
        Delete this Resource. True on success.
        """
        if getattr(self, 'id', None) is None:
            raise ValueError('%s cannot be deleted before it is saved' % self)
        conn = self._get_conn(using)
        path = self.Meta.detail_path.format(id=self.id)
        resp = conn.delete(conn.build_absolute_url(path))
        if resp.status_code in self._delete_success_codes:
            return True
        else:
            raise ApiError(resp.status_code, resp.text)


class ReadWritable(Creatable, Readable, Updateable, Deletable):
    pass



# -*- coding: utf-8 -*-

from __future__ import absolute_import, print_function, unicode_literals


class ProspyrException(Exception):
    pass


class ApiError(ProspyrException):
    def __str__(self):
        return 'HTTP %s: %s' % self.args

    def __unicode__(self):
        return 'HTTP %s: %s' % self.args


class MisconfiguredError(ProspyrException):
    pass


class ValidationError(ProspyrException):
    pass



# -*- coding: utf-8 -*-

from __future__ import absolute_import, print_function, unicode_literals

from logging import getLogger

from marshmallow import fields
from six import with_metaclass

from prospyr import connection, exceptions, mixins, schema
from prospyr.search import ResultSet

logger = getLogger(__name__)


class Manager(object):

    def get(self, id):
        instance = self.resource_cls()
        instance.id = id
        instance.read(using=self.using)
        return instance

    def __get__(self, instance, cls):
        if instance:
            raise AttributeError(
                'You can\'t access this {manager_cls} from a resource '
                'instance; Use {resource_cls}.objects instead'.format(
                    resource_cls=type(instance).__name__,
                    manager_cls=type(self).__name__
                )
            )
        self.resource_cls = cls
        self.using = 'default'
        return self

    def use(self, name):
        self.using = name
        return self

    def all(self):
        return self.filter()

    def filter(self, **query):
        return ResultSet(resource_cls=self.resource_cls,
                         using=self.using).filter(**query)

    def order_by(self, field):
        return ResultSet(resource_cls=self.resource_cls,
                         using=self.using).order_by(field)


class ResourceMeta(type):
    """
    Metaclass of all Resources.
    """
    def __new__(cls, name, bases, attrs):
        super_new = super(ResourceMeta, cls).__new__

        # only do metaclass tomfoolery for resource *subclasses*
        parents = [b for b in bases if issubclass(b, Resource)]
        if not parents:
            return super_new(cls, name, bases, attrs)

        # move marshmallow fields to a new Schema subclass on cls.Meta
        schema_attrs = {}
        for attr, value in list(attrs.items()):
            if isinstance(value, fields.Field):
                schema_attrs[attr] = attrs.pop(attr)
        schema_cls = type(
            '%sSchema' % name,
            (schema.TrimSchema, ),
            schema_attrs
        )
        attrs['Meta'].schema = schema_cls()

        return super_new(cls, name, bases, attrs)


class Resource(with_metaclass(ResourceMeta)):

    objects = Manager()

    def __init__(self, **data):
        self._set_fields(data)

    def validate(self):
        schema = self.Meta.schema
        attrs = set(dir(self)) & set(schema.declared_fields)
        data = {k: getattr(self, k) for k in attrs}
        errors = self.Meta.schema.validate(data)
        if errors:
            raise exceptions.ValidationError(errors)

    @classmethod
    def from_api_data(cls, data):
        """
        Alternate constructor. Build instance from ProsperWorks API data.
        """
        data, errors = cls.Meta.schema.load(data)
        if errors:
            raise exceptions.ValidationError(
                'ProsperWorks delivered data which does not agree with the '
                'local prospyr schema. Errors encountered: %s' % repr(errors)
            )
        instance = cls()
        instance._set_fields(data)
        return instance

    def _data_from_resp(self, resp):
        data, errors = self.Meta.schema.load(resp.json())
        if errors:
            raise exceptions.ValidationError(
                'ProsperWorks delivered data which does not agree with the '
                'local prospyr schema. Errors encountered: %s' % repr(errors)
            )
        return data

    def __repr__(self):
        classname = type(self).__name__
        friendly = str(self)
        return '<%s: %s>' % (classname, friendly)

    def __str__(self):
        if hasattr(self, 'name'):
            return self.name
        elif hasattr(self, 'id'):
            return str(self.id)
        return '(unsaved)'

    def _get_conn(self, using):
        return connection.get(using)

    def _set_fields(self, data):
        """
        Without validating, write `data` onto the fields of this Resource.
        """
        for field, value in data.items():
            setattr(self, field, value)

    @property
    def _raw_data(self):
        schema = self.Meta.schema
        data, errors = schema.dump(self)
        if errors:
            raise exceptions.ValidationError(
                'Could not serialize %s data: %s' % (self, repr(errors))
            )

        return data


class Related(object):
    """
    Behave as a related object when an attribute of a Resource.
    """

    def __init__(self, related_cls):
        self.related_cls = related_cls

    def __get__(self, instance, cls):
        if instance is None:
            return self
        attr = self.find_parent_attr(type(instance))
        id = getattr(instance, '%s_id' % attr)
        if id is None:
            return None
        return self.related_cls.objects.get(id=id)

    def __set__(self, instance, value):
        attr = self.find_parent_attr(type(instance))
        if not isinstance(value, self.related_cls):
            raise ValueError(
                '`{value}` must be an instance of `{cls}`'
                .format(value=value, cls=self.related_cls)
            )
        if not value.id:
            raise ValueError(
                '`{value}` can\'t be assigned without an `id` attribute.'
                .format(value=value)
            )
        setattr(instance, '%s_id' % attr, value.id)

    def find_parent_attr(self, parent_cls):
        for attr in dir(parent_cls):
            if getattr(parent_cls, attr) is self:
                return attr
        else:
            raise AttributeError('Cannot find self')


class User(Resource, mixins.Readable):

    id = fields.Integer()
    name = fields.String(required=True)
    email = fields.Email(required=True)

    class Meta(object):
        # schema = schema.UserSchema()
        search_path = 'users/search/'
        detail_path = 'users/{id}/'

    def __str__(self):
        return '{self.name} ({self.email})'.format(self=self)


class Company(Resource, mixins.Readable):

    class Meta(object):
        search_path = 'companies/search/'
        detail_path = 'companies/{id}/'
        order_fields = {
            'name',
            'city',
            'state',
            'assignee',
            'inactive_days',
            'last_interaction',
            'interaction_count',
            'date_created',
            'date_modified',
        }

    id = fields.Integer()
    name = fields.String(required=True)
    address = fields.Nested(schema.AddressSchema, allow_none=True)
    assignee_id = fields.Integer(allow_none=True)
    assignee = Related(User)
    contact_type_id = fields.Integer(allow_none=True)
    details = fields.String(allow_none=True)
    email_domain = fields.String(allow_none=True)
    phone_numbers = fields.Nested(schema.PhoneNumberSchema(many=True))
    socials = fields.Nested(schema.SocialSchema(many=True))
    tags = fields.List(fields.String)
    date_created = schema.Unix()
    date_modified = schema.Unix()
    websites = fields.Nested(schema.CustomFieldSchema(many=True))
    custom_fields = fields.Nested(schema.WebsiteSchema(many=True))


class Person(Resource, mixins.Creatable, mixins.Readable, mixins.Deletable,
             mixins.Updateable):

    class Meta(object):
        create_path = 'people/'
        search_path = 'people/search/'
        detail_path = 'people/{id}/'
        order_fields = {
            'name',
            'city',
            'state',
            'assignee',
            'inactive_days',
            'last_interaction',
            'interaction_count',
            'date_created',
            'date_modified',
        }

    id = fields.Integer()
    name = fields.String(required=True)
    address = fields.Nested(
        schema.AddressSchema,
        allow_none=True
    )
    assignee_id = fields.Integer(allow_none=True)
    assignee = Related(User)
    company_id = fields.Integer(allow_none=True)
    company = Related(Company)
    company_name = fields.String(allow_none=True, load_only=True)
    contact_type_id = fields.Integer(allow_none=True)
    details = fields.String(allow_none=True)
    emails = fields.Nested(
        schema.EmailSchema,
        many=True,
        required=True,
    )
    phone_numbers = fields.Nested(schema.PhoneNumberSchema, many=True)
    socials = fields.Nested(schema.SocialSchema, many=True)
    tags = fields.List(fields.String)
    title = fields.String(allow_none=True)
    date_created = schema.Unix()
    date_modified = schema.Unix()
    websites = fields.Nested(schema.CustomFieldSchema, many=True)
    custom_fields = fields.Nested(schema.WebsiteSchema, many=True)



# -*- coding: utf-8 -*-

from __future__ import absolute_import, print_function, unicode_literals

from collections import Mapping, namedtuple

import arrow
from arrow.parser import ParserError
from marshmallow import (Schema, ValidationError, fields, post_dump, post_load,
                         pre_dump)


class TrimSchema(Schema):
    """
    A schema which does not dump "empty" fields.

    ...where empty is defined as None for value fields, and an empty list for
    collection fields.

    This is in line with the behaviour ProsperWorks expects.
    """

    @post_dump
    def clean_empty(self, data):
        to_clean = []
        for key, value in data.items():
            collection = getattr(self.fields[key], 'many', False)
            if collection and value == []:
                to_clean.append(key)
            elif value is None:
                to_clean.append(key)

        for key in to_clean:
            data.pop(key)

        return data


class NamedTupleSchema(Schema):
    """
    (De)serialise to namedtuple instead of dict
    """
    def __init__(self, *args, **kwargs):
        super(NamedTupleSchema, self).__init__(*args, **kwargs)
        name = type(self).__name__.replace('Schema', '')
        fields = self.declared_fields.keys()
        self.namedtuple_class = namedtuple(name, fields)

    @post_load
    def to_namedtuple(self, data):
        return self.namedtuple_class(**data)

    @pre_dump
    def from_namedtuple(self, obj):
        if not isinstance(obj, Mapping):
            return obj._asdict()
        return obj


class Unix(fields.Field):
    """
    datetime.datetime <-> unix timestamp
    """
    def _serialize(self, value, attr, obj):
        try:
            return arrow.get(value).timestamp
        except ParserError as ex:
            raise ValidationError(ex)

    def _deserialize(self, value, attr, obj):
        try:
            return arrow.get(value).datetime
        except ParserError as ex:
            raise ValidationError(ex)


class EmailSchema(NamedTupleSchema):
    email = fields.Email()
    category = fields.String()


class WebsiteSchema(Schema):
    website = fields.Url()
    category = fields.String()


class SocialSchema(Schema):
    website = fields.Url()
    category = fields.String()


class PhoneNumberSchema(Schema):
    number = fields.String()
    category = fields.String()


class CustomFieldSchema(Schema):
    custom_field_definition_id = fields.Integer()
    value = fields.String()  # TODO base this on field definition


class AddressSchema(Schema):
    street = fields.String(allow_none=True)
    city = fields.String(allow_none=True)
    state = fields.String(allow_none=True)
    postal_code = fields.String(allow_none=True)
    country = fields.String(allow_none=True)



# -*- coding: utf-8 -*-

from __future__ import absolute_import, print_function, unicode_literals

from collections import namedtuple
from logging import getLogger

import arrow

logger = getLogger(__name__)
CacheEntry = namedtuple('CacheEntry', 'value,created,max_age')


class InMemoryCache(object):
    """
    An in-memory cache. Keys are expired by count and age.
    """

    def __init__(self, size=500):
        self._cache = {}
        self._size = size

    def meta(self, key):
        return self._cache[key]

    def set(self, key, value, max_age=0):
        now = arrow.utcnow().timestamp
        entry = CacheEntry(value=value, created=now, max_age=max_age)
        logger.debug('%s added to cache', key)
        self._cache[key] = entry
        self._maintenance()
        return True

    def get(self, key):
        self._maintenance()
        try:
            entry = self._cache[key]
            logger.debug('Cache hit for %s', key)
            return entry.value
        except KeyError:
            logger.debug('Cache miss for %s', key)
            return None

    def clear(self, key):
        if key in self._cache:
            logger.debug('Cleared %s', key)
            self._cache.pop(key)
        return True

    def _maintenance(self):
        # expire any keys older than max_age
        to_expire = []
        now = arrow.utcnow().timestamp
        for key, entry in self._cache.items():
            too_old = (
                entry.max_age and
                entry.created + entry.max_age <= now
            )
            if too_old:
                to_expire.append(key)
        for key in to_expire:
            logger.debug('%s has expired', key)
            del self._cache[key]

        # if too many keys, repeatedly expire oldest
        excess_size = len(self._cache) - self._size
        if excess_size > 0:
            key_ages = [(created, key)
                        for key, (_, created, _) in self._cache.items()]
            key_ages = sorted(key_ages)
            for _, key in key_ages[:excess_size]:
                logger.debug('Cache too full, evicted %s', key)
                del self._cache[key]


class NoOpCache(object):
    """
    A cache class which doesn't cache anything.
    """

    def meta(self, key):
        return None

    def set(self, key, value, max_age=0):
        return True

    def get(self, key):
        return None

    def clear(self, key):
        return True



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
from codecs import open

from setuptools import setup

# README.rst contents
here = os.path.abspath(os.path.dirname(__file__))
with open(os.path.join(here, 'README.rst'), encoding='utf-8') as f:
    readme = f.read()


# project version
context = {}
with open(os.path.join(here, 'prospyr/version.py')) as f:
    exec(f.read(), context)
    version = '.'.join(str(p) for p in context['VERSION'])


# requirements
def just_packages(path):
    with open(path, encoding='utf-8') as f:
        reqs = list(f)
    return [req.split('==')[0] for req in reqs
            if '==' in req and not req.startswith('#')]
requirements = just_packages(os.path.join(here, 'requirements'))
dev_requirements = just_packages(os.path.join(here, 'dev-requirements'))


setup(
    name='prospyr',
    description='ProsperWorks client library',
    long_description=readme,
    version=version,
    url='https://github.com/salespreso/prospyr/',
    author='Ben Graham',
    author_email='ben.graham@salespreso.com',
    license='MIT',
    classifiers=[
        'Intended Audience :: Developers',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
    keywords='ProsperWorks',
    packages=['prospyr'],
    install_requires=requirements,
    extras_require={'dev': dev_requirements},
    test_suite='nose.core.collector',
    tests_require=dev_requirements,
)



# -*- coding: utf-8 -*-

from __future__ import division, print_function, unicode_literals

import importlib
import re
from datetime import timedelta


def _parts(string):
    if '-' in string and '_' not in string:
        parts = string.split('-')
    elif '_' in string and '-' not in string:
        parts = string.split('_')
    else:
        parts = re.findall(r'[A-Z]+[^A-Z]*', string)
    if not parts:
        parts = [string]
    return [p.lower() for p in parts]


def to_snake(string):
    """
    Converts `string` to snake_case.
    """
    return '_'.join(_parts(string))


def to_kebab(string):
    """
    Converts `string` to kebab-case.
    """
    return '-'.join(_parts(string))


def to_camel(string):
    """
    Converts `string` to CamelCase.
    """
    return ''.join(p.title() for p in _parts(string))


def import_dotted_path(path):
    """
    Return the module or name at `path`.
    """
    parts = path.split('.')
    args = []
    found = None

    for l in range(1, len(parts)):
        iterpath = '.'.join(parts[:l])
        attr = parts[l]
        args.append((iterpath, attr))

    for path, attr in args:
        print(path, attr)
        try:
            found = importlib.import_module(path)
            print('found:', found)
        except ImportError:
            pass

        if hasattr(found, attr):
            found = getattr(found, attr)

    if found is None:
        raise ImportError('Cannot import %s', path)

    return found


def seconds(days=0, seconds=0, microseconds=0, milliseconds=0, minutes=0,
            hours=0, weeks=0):
    """
    Convenient time-unit-to-seconds converter. A thin wrapper around timedelta.
    """
    td = timedelta(days=days, seconds=seconds, microseconds=microseconds,
                   milliseconds=milliseconds, minutes=minutes, hours=hours,
                   weeks=weeks)
    return td.total_seconds()
